/*
  yasep/defora/gen_asm.js created by whygee at f-cpu.org
  This file is distributed under terms of the Affero GPL v3 license or later, see http://yasep.org
  created 2011-09-25 from yasep/tool/test_opcodes.js
  version lun. nov.  7 13:21:06 CET 2011 : changed name, changed for use with filefox

  This menu item creates a dump of the opcodes in all forms for defora's assembler.
    2 outputs are provided :
       - an assembly testbench/regression file, with source/cleartext and assembled/hexa versions
       - another dump for creating the assembler's parser rules.
*/

init_obj.push(DEF_ASM={
  win:null,
  Hexa:"", S:"", pad:"",
  // start() creates other variables

  add_message: function(col,msg){
    if ((col=="blue") || (col=="green"))
      return;
    DEF_ASM.S +=' ###### '+msg+'\n';
  },

  test_opcode: function(x) {
    DEF_ASM.S+='\t'+pad_chars(x,24)+'; ';
    if (Y.ASM.encode_instruction(x,DEF_ASM.ASM)) {
      var e=DEF_ASM.ASM.encoded_instruction,
          s=DEF_ASM.ASM.encoded_size_bytes;
      while (s--) { // displays every byte, little endian style
        DEF_ASM.S+=' '+(e&255);
        e>>=8;
      }
      DEF_ASM.S +='\n';
    }
    else {
      DEF_ASM.add_message('red',' ERROR !');
      DEF_ASM.errnum++;
    }
  },

  describe_opcode: function(name, index, form, flags){
    var s='/* '+pad_chars(Y.TableForm[form],8)+' */ { '+pad_chars('"'+name+'",',10);
    s+=pad_chars(index+',', 5)+'  AOT_SND,   AOT_SI4,  AOT_DST';
    DEF_ASM.pad=",  AOT_PAD";
    return s;
  },

  start: function() {
    var theme="doc_gryl";
    // (re-)initialise important variables
    DEF_ASM.msg_number=0;
    DEF_ASM.errnum=0;
    DEF_ASM.sorted_opcodes=[];
    DEF_ASM.max_opcode=0;
    DEF_ASM.cur_opcode=0;
    var d=new Date().toGMTString(); // today's date
    DEF_ASM.S='; DeforaOS/Apps/Devel/src/asm/test/yasep.S\n'
       +'; Assembly language dump generated to test the Defora assembler\n'
       +'; Target : yasep (yasep16 and yasep32 will come soon)\n'
       +'; Generated by yasep/defora/gen_asm.js on '+d+'\n'
       +'; Bytes are little endian order.\n\n.text\n';

   DEF_ASM.Hexa=
'/* DeforaOS/Apps/Devel/src/asm/src/arch/yasep.ins\n'+
' Generated by yasep/defora/gen_asm.js on '+d+'\n'+
' (yasep16/yasep32 will come later) */\n\n';

    DEF_ASM.ASM = new Y.ASM_context(DEF_ASM.add_message);

    // prepare the sorted list of opcodes and aliases
    for (var name in Y.table_opcode) {
      if (Y.table_opcode[name].forms>0)
        DEF_ASM.sorted_opcodes[DEF_ASM.max_opcode++]=name;
    }
    DEF_ASM.sorted_opcodes=DEF_ASM.sorted_opcodes.sort();

    while (DEF_ASM.cur_opcode<DEF_ASM.max_opcode) {
      var name=DEF_ASM.sorted_opcodes[DEF_ASM.cur_opcode++];
      var index=Y.table_opcode[name].val;
      var instr=Y.opcode_table[index];

      DEF_ASM.S+='\n; '+int2hex(index)+' = '+name+' : '+instr.description+'\n';
      DEF_ASM.Hexa+='/**** '+name+' ****/\n';
      // Scan the forms list
      var f=instr.forms;
      var i=1;

      do {
        if (f & i) {
          var w=Y.TableFormFunction[i](instr.flags); // random operands
          if (w=='') // no operands so don't add a spacer that will bork the instruction
            w=instr.name;
          else 
            w=instr.name+" "+w;

          DEF_ASM.test_opcode(w);

          var z=DEF_ASM.describe_opcode(name, index, i, instr.flags);
          DEF_ASM.Hexa+=  z+',  AOT_NONE,     AOT_NONE'+DEF_ASM.pad+' },\n';

          if ((i & Y.FORMS_EXTENDABLE)&&((instr.flags & Y.NO_CONDITION)==0)) {
            DEF_ASM.test_opcode(w+' '+Y.randomCondition());
            DEF_ASM.Hexa+=z+',  Y_COND_ALONE, AOT_NONE'+DEF_ASM.pad+' },\n'
                        + z+',  Y_COND_REG,   OP_R    '+DEF_ASM.pad+' },\n'
                        + z+',  Y_COND_IM4,   IMM4    '+DEF_ASM.pad+' }, /* not implemented yet */\n';
          }
        }
        i<<=1;
      } while (i <= f);
    } // while EF_ASM.cur_opcode<DEF_ASM.max_opcode

    if (DEF_ASM.errnum > 0)
      theme="doc_red";

    DEF_ASM.Hexa = DEF_ASM.Hexa.slice(0,-2); // Trim the last ",\n"

    var pre = document.createElement('pre');
    pre.innerHTML='<button id="save_S">Save yasep.S</button>'
       +' &nbsp; <button id="save_h">Save yasep.h</button><br>'
       +DEF_ASM.S+"<hr>"+DEF_ASM.Hexa;
    getById("save_S",null,pre).onclick=function(){
      savefile.save("yasep.S",DEF_ASM.S);
    };
    getById("save_h",null,pre).onclick=function(){
      savefile.save("yasep.h",DEF_ASM.Hexa);
    };

    return(DEF_ASM.win= ygwm.new_window( "Defora ASM files", null, pre,
       550, 500, 350, 70, false, true, true, true, false, theme));
  },

  init_obj: function() {
    DEF_ASM.menuItem = menu.subMenuItem(menu.mainMenuItem("Dev"), "asm&nbsp;test", "def/asm");
  }
});
