/*
yasep/gui-js/winman.js
This file is distributed under terms of the Affero GPL v3 license or later, see http://yasep.org
created ven. sept.  9 22:00:18 CEST 2011 by whygee@f-cpu.org
version lun. sept. 12 07:57:43 CEST 2011
version mar. sept. 13 09:21:06 CEST 2011 aliases, removed createCallback
version lun. sept. 19 08:20:49 CEST 2011 removed aliases
version sam. oct.  15 05:51:37 CEST 2011 adding focus on argument
version lun. nov.  7 16:42:59   CET 2011 changed register_window to register_key

This file uses a list of window keys to keep keyed windows open in only one instance.
It adds other fancy stuff too, such as patchInnerLinks or hiddenText ...

Dependencies : ygwm.js
Use : register_key() is called by all the files that create application windows or documentation,
  so it is included after ygwm and before the menu and the apps.
Note : the window creation function (given to register_key) must return the id of the ygwm_div element created.
*/

winman={
  // debug

  window_keys:[],   // stores either a string (when alias)
  // or a key object (key name, on/off, creator function, ygwm id)

  keys_list:[],  // simple array of the above

  // Add the key and function to the window keys table
  // new version : the whole object is passed, not just the function,
  //  so the key is added to the object
  register_key: function(key,f) {
    if ((typeof key != "string") || (typeof f != "function"))
      return alert("register_key : wrong argument type");
    if (winman.window_keys[key]) {
      if (key.substr(0,4) == "ISM/")
        return;  // allow several redefinitions of the same key if we register an ISM key
      else
        alert("key "+key+" already registered");
    };
    winman.window_keys[key]={func:f, on:false};
    winman.keys_list.push(key);
  },

  activate_window: function(s) {
    var key, t, u, v, v2, w;
    // when called from a A tag, "s" is not a string so we examine the href
    if (typeof s != "string") {
      t=this.href;
      if (t){
        u=t.indexOf("#!");
        s = t.slice(u+2); // strip the link of the absolute path
      }
      else return; // invalid parameter type
    }

    // split s into a key and arguments
    u=s.indexOf('#');
    if (u<0) {
      key=s;
      s='';
    } else {
      key=s.slice(0,u);
      s=s.slice(u+1);
    }

    v=key;
    w=winman.window_keys;

    // scan the eventual alias linked list
    do{
      if (!w[v]) {
        alert("feature "+key+" ("+v+") is not (yet) created :-/");
        return false;
      }
      v2=v;
      v=w[v];
    } while (typeof v=="string");

    if (v.on==false){ // Window not created : create it
      v.win = v.func(v2,key); // This way, one object can handle several keys !
      v.win.eraseCallback=function() { // closure
        v.on=false;
      };
      v.on=true;
    }
    else { // just display the existing window
      ygwm.focus_window(null,v.win);
      ygwm.center_window(v.win);
    }

    if(s != '') {
     if (v.win.trigger)
       v.win.trigger(s);
     else
       document.location.href="#"+s;
    }
  },

  patchInnerLinks: function(base){
    if (!base)
      return;

    // Scan all the <a> tags
    var s,t,
     u=base.getElementsByTagName('a'),
     v="";
    for (var i in u){
      if(u[i].className=="asm"){
        u[i].href="#!tool/asm#a?"+u[i].innerHTML;
        u[i].onclick=winman.activate_window;
        u[i].className="asm_active";
      }
      else {
        if(u[i].className=="disasm"){
          u[i].href="#!tool/asm#d?"+u[i].innerHTML;
          u[i].onclick=winman.activate_window;
          u[i].className="disasm_active";
        }
        else {
          if (u[i].className=="opcode"){
            u[i].href="#!ISM/"+u[i].innerHTML;
            u[i].onclick=winman.activate_window;
            u[i].className="opcode_active";
          }
          else {
            if(u[i].href) {
              t=u[i].href;
              s=t.indexOf("#!");
              if (s >= 0) {
                t=t.slice(s+2);
                s=t.split("#");
                if(winman.window_keys[s[0]]) {
                  u[i].onclick=winman.activate_window;
                  u[i].className="innerLink";
                  v+=s[0];
                }
              }
            }
          }
        }
      }
    }
  }

};

// used by the applications to fetch hidden data from the .html
function hiddenText(key,base){
  return getById("hidden"+key,"SPAN",base).innerHTML;
}

wa=winman.activate_window; // HTML ALIAS
